from .engines import *


__title__ = 'search_engines'
__version__ = '0.5'
__author__ = 'Tasos M. Adamopoulos'

__all__ = [
    'Google', 
    'Bing', 
    'Yahoo', 
    'Aol', 
    'Duckduckgo', 
    'Startpage', 
    'Dogpile', 
    'Ask', 
    'Mojeek', 
    'Qwant',
    'Torch'
]
